/*
    OTP4U 0.8.0 - One Time Pad for you
    Copyright (C) 2003 Pierre Blanc
    Pierre Blanc: blanc_teutoburgo@yahoo.it
    http://www.teutoburgo.tk
    http://it.geocities.com/teutoburgo

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    or go to      http://www.gnu.org/copyleft/gpl.html
*/

package tk.teutoburgo.otp4u.crypt;

import java.util.*;
import java.io.*;
import tk.teutoburgo.otp4u.constants.*;

/**
 * Title:        OTP4U
 * Description:
 * Copyright:    Copyright (c) 2002
 * Company:      Teutoburgo
 * @author Pierre Blanc
 * @version 1.0
 */

public class KM_Xor extends KeyManager {

    String ERROR_XOR=Constants.ERROR_XOR;

    public KM_Xor() {
    }

    public KM_Xor(String home, Boolean isGui) {
	super(home, isGui);
    }
    public void writeRandomKey() throws IOException, FileNotFoundException{
        byte[] pubLeft=null, publicKeyBytes = io.readFile(OTP4U_HOME+
	  PUBLIC_KEY + PUBLIC_KEY_FILE), oldKey4Key = io.readFile(
          OTP4U_HOME + RANDOM_KEY +KEY4KEY + KEY4KEY_FILE), pubRight=null;
	int i, pLen=publicKeyBytes.length, n=pLen/2;
        if(pLen==2*oldKey4Key.length){
          pubLeft = new byte[n];
          pubRight = new byte[n];
          for(i=0; i<n; i++){
              pubLeft[i] = publicKeyBytes[i];
              pubRight[i] = publicKeyBytes[i+n];
          }
          byte[] randomLeft = xor.decrypt(pubLeft, oldKey4Key);
          byte[] auto = autoKey(randomLeft, oldKey4Key);
          byte[] randomRight = xor.decrypt(pubRight, auto);
          randomRight = getKey4Message(oldKey4Key, randomRight);
          io.writeFile(OTP4U_HOME + RANDOM_KEY + RANDOM_KEY_FILE,
                       randomRight);
          io.writeFile(OTP4U_HOME +RANDOM_KEY+ KEY4KEY +
                       KEY4KEY_FILE , randomLeft);
        } else
              mp.sPrintLog(ERROR_XOR);
    }
    public void writePublicKey() throws IOException, FileNotFoundException{
        byte[][] entropyFiles = io.readRandomKey(OTP4U_HOME + ENTROPY);
	byte[] randomKeyBytes = this.compactEntropyFiles(entropyFiles);
	byte[] oldKey4Key = io.readFile(OTP4U_HOME + RANDOM_KEY +KEY4KEY +
					KEY4KEY_FILE);
        if(randomKeyBytes.length==2*oldKey4Key.length){
          byte[] publicKeyBytes=   xor.encrypt(randomKeyBytes,
                 getKey4Key(randomKeyBytes, oldKey4Key));
          io.writeFile(OTP4U_HOME+ PUBLIC_KEY + PUBLIC_KEY_FILE ,
                       publicKeyBytes);
          writeRandomAndK4K(randomKeyBytes, oldKey4Key);
        } else
              mp.sPrintLog(ERROR_XOR);
    }
    public byte[] getKey4Key(byte[] y, byte[] oldKey4Key) throws IOException {
	byte[] oldK4KAuto = autoKey(oldKey4Key, y);
	int k4kLen = oldKey4Key.length*2;
	byte[] key4key = new byte[k4kLen];
	for(int i=0; i< k4kLen/2; i++){
	    key4key[i]=oldKey4Key[i];
	    key4key[i+k4kLen/2]=oldK4KAuto[i];
	}
	return key4key;
    }
    public void writeRandomAndK4K(byte[] randomKeyBytes, byte[] oldKey4Key){
	try{
	    int i=0, rLen=randomKeyBytes.length;
	    byte[] halfRandom=new byte[rLen/2];
	    byte[] newKey4Key = new byte[rLen/2];
	    for(i=0; i< rLen/2; i++){
		newKey4Key[i]=randomKeyBytes[i];
		halfRandom[i]=randomKeyBytes[i+rLen/2];
	    }
	    halfRandom = getKey4Message(oldKey4Key, halfRandom);
	    io.writeFile(OTP4U_HOME + RANDOM_KEY + RANDOM_KEY_FILE,
			 halfRandom);
	    io.writeFile(OTP4U_HOME +RANDOM_KEY+ KEY4KEY +
			 KEY4KEY_FILE , newKey4Key);
	}catch(Exception e){
	    e.printStackTrace();
	}
    }
    /**
     * Returns a new byte[] of the same length of the parameters
     * @param  x,y random byte[]
     */
    public byte[] autoKey(byte[] x, byte[] y){
	int i,n=x.length;
	byte[] auto =new byte[n], temp=new byte[n];
	for (i=0; i<n; i++){
	    temp[i] = y[n-1-i];
	}
	return xor.encrypt(x, temp);
    }
    /*
     * Gets from the parameters the array of key4message, obtained from a
     * permutation over the blocks of r0R
     * @param key4key used to generate a random permutation
     * @param r0R the random bytes to be permuted
     * @return The array that will be written in key4message
     */
    public byte[] getKey4Message(byte[] key4key, byte[] r0R){
	int BLOCKS=64, len=r0R.length, i, BLOCK_LENGTH=len/BLOCKS, MAX=1000,
	count=0;
	Byte b;
	int[] perm=new int[BLOCKS];
	byte[] sorted=new byte[BLOCKS], origin=new byte[BLOCKS],
	source=key4key,	dest=new byte[len];
	ArrayList bytes=new ArrayList();

	for(i=0; i<BLOCKS+MAX && count<BLOCKS; i++){
	    b=new Byte(key4key[i]);
	    if(bytes.indexOf(b)==-1){
		bytes.add(b);
		count++;
	    }
	}
	//		System.out.println("count "+count+"  i="+i);
	if(count<BLOCKS){
	    mp.sPrintLog("Error: probably key4key isn't random.");
	    System.exit(0);
	}else{
	    Object[] temp=bytes.toArray();
	    for(i=0; i<BLOCKS; i++){
		sorted[i]=((Byte)temp[i]).byteValue();
		origin[i]=sorted[i];
	    }
	    Arrays.sort(sorted);
	    for(i=0; i<BLOCKS; i++){
		perm[i]=Arrays.binarySearch(sorted, origin[i]);
		/*System.out.print("i="+i+" k4k="+origin[i]+" so="+sorted[i]+
		" perm="+perm[i]+"   ");*/
	    }
	    for(i=0; i<BLOCKS; i++){
		System.arraycopy(source, perm[i]*BLOCK_LENGTH, dest,
				 i*BLOCK_LENGTH, BLOCK_LENGTH);
	    }
	}

	return dest;
    }

}
